#include "hwsim.h"

#if defined(_WIN32)
typedef UINT32 uint32_t;
typedef BOOL bool_t;
#endif

#if !defined(_WIN32)
# include <stdint.h>
# include <stdio.h>
# include <stdlib.h>
# include <string.h>
# define _tcscpy_s(dst, n, src) strcpy(dst, src);
# define _stprintf_s snprintf
# define _T(x) x
# define TRUE (1)
# define FALSE (0)
typedef int bool_t;
typedef char TCHAR;
#endif

#define ARRAY_LENGTH(x) (sizeof(x) / sizeof((x)[0]))

static const unsigned int g_numDevice = 10;

static bool_t
bDeviceValid(
  ADMXRC3_HANDLE hCard,
  unsigned int& index) {
  if (hCard != ADMXRC3_HANDLE_INVALID_VALUE && (unsigned int)hCard <= g_numDevice) {
    index = (unsigned int)hCard - 1U;
    return TRUE;
  } else {
    return FALSE;
  }
}

ADMXRC3_EXPORT ADMXRC3_STATUS
ADMXRC3_CALLING_CONVENTION
ADMXRC3_Close(
  ADMXRC3_HANDLE hCard) {
  unsigned int index;

  if (!bDeviceValid(hCard, index)) {
    return ADMXRC3_INVALID_HANDLE;
  } else {
    return ADMXRC3_SUCCESS;
  }
}

ADMXRC3_EXPORT ADMXRC3_STATUS
ADMXRC3_CALLING_CONVENTION
ADMXRC3_GetCardInfoEx(
  ADMXRC3_HANDLE hCard,
  ADMXRC3_CARD_INFOEX* pCardInfoEx) {
  unsigned int index;
  int model;

  if (!bDeviceValid(hCard, index)) {
    return ADMXRC3_INVALID_HANDLE;
  };

  memset(pCardInfoEx, 0, sizeof(*pCardInfoEx));
  switch (index) {
  case 0:
    model = ADMXRC3_MODEL_ADMXRC2;
    pCardInfoEx->NumTargetFpga = 1;
    pCardInfoEx->NumWindow = 3;
    break;

  case 1:
    return ADMXRC3_UNEXPECTED_ERROR;

  case 6:
  case 7:
  case 8:
  case 9:
    model = (unsigned int)ADMXRC3_MODEL_ADPEXRC5T + index - 2;
    pCardInfoEx->NumTargetFpga = index - 6;
    pCardInfoEx->NumWindow = index - 5;
    break;

  default:
    model = (unsigned int)ADMXRC3_MODEL_ADPEXRC5T + index - 2;
    pCardInfoEx->NumTargetFpga = 1;
    pCardInfoEx->NumWindow = 4;
    break;
  }

  switch (index) {
  case 0:
    pCardInfoEx->NumSensor = 0;
    break;

  default:
    pCardInfoEx->NumSensor = g_numDevice - index + 3;
    break;
  }

  pCardInfoEx->Model = (ADMXRC3_MODEL_TYPE)model;
  pCardInfoEx->SerialNumber = index + 100;

  return ADMXRC3_SUCCESS;
}

ADMXRC3_EXPORT ADMXRC3_STATUS
ADMXRC3_CALLING_CONVENTION
ADMXRC3_GetFpgaInfo(
  ADMXRC3_HANDLE hCard,
  unsigned int targetIndex,
  ADMXRC3_FPGA_INFO* pFpgaInfo) {
  unsigned int index;

  if (!bDeviceValid(hCard, index)) {
    return ADMXRC3_INVALID_HANDLE;
  };

  memset(pFpgaInfo, 0, sizeof(*pFpgaInfo));
  switch (index) {
  case 0:
    _tcscpy_s(pFpgaInfo->Identifier, ARRAY_LENGTH(pFpgaInfo->Identifier), _T("2v6000ff1152"));
    break;

  case 1:
    return ADMXRC3_UNEXPECTED_ERROR;

  case 6:
  case 7:
  case 8:
  case 9:
    if (index == 8 && targetIndex == 0) {
      return ADMXRC3_UNEXPECTED_ERROR;
    }
    if (index == 9 && targetIndex == 2) {
      return ADMXRC3_UNEXPECTED_ERROR;
    }
    _tcscpy_s(pFpgaInfo->Identifier, ARRAY_LENGTH(pFpgaInfo->Identifier), _T("6vlx444tff9999"));
    break;

  default:
    _tcscpy_s(pFpgaInfo->Identifier, ARRAY_LENGTH(pFpgaInfo->Identifier), _T("7vlx333tff5000"));
    break;
  }
  pFpgaInfo->Present = TRUE;

  return ADMXRC3_SUCCESS;
}

ADMXRC3_EXPORT const TCHAR*
ADMXRC3_CALLING_CONVENTION
ADMXRC3_GetStatusString(
  ADMXRC3_STATUS code,
  bool_t bShort) {
  static TCHAR buffer[128];

  if (bShort) {
    switch (code) {
    case ADMXRC3_SUCCESS:
      return _T("ADMXRC3_SUCCESS");

    case ADMXRC3_INTERNAL_ERROR:
      return _T("ADMXRC3_INTERNAL_ERROR");

    case ADMXRC3_UNEXPECTED_ERROR:
      return _T("ADMXRC3_UNEXPECTED_ERROR");

    case ADMXRC3_BAD_DRIVER:
      return _T("ADMXRC3_BAD_DRIVER");

    case ADMXRC3_NO_MEMORY:
      return _T("ADMXRC3_NO_MEMORY");

    case ADMXRC3_ACCESS_DENIED:
      return _T("ADMXRC3_ACCESS_DENIED");

    case ADMXRC3_DEVICE_NOT_FOUND:
      return _T("ADMXRC3_DEVICE_NOT_FOUND");

    case ADMXRC3_FILE_NOT_FOUND:
      return _T("ADMXRC3_FILE_NOT_FOUND");

    case ADMXRC3_HARDWARE_ERROR:
      return _T("ADMXRC3_HARDWARE_ERROR");

    case ADMXRC3_FPGA_MISMATCH:
      return _T("ADMXRC3_FPGA_MISMATCH");

    case ADMXRC3_INVALID_BITSTREAM:
      return _T("ADMXRC3_INVALID_BITSTREAM");

    case ADMXRC3_INVALID_BUFFER:
      return _T("ADMXRC3_INVALID_BUFFER");

    case ADMXRC3_INVALID_FLAG:
      return _T("ADMXRC3_INVALID_FLAG");

    case ADMXRC3_INVALID_FREQUENCY:
      return _T("ADMXRC3_INVALID_FREQUENCY");

    case ADMXRC3_INVALID_HANDLE:
      return _T("ADMXRC3_INVALID_HANDLE");

    case ADMXRC3_INVALID_INDEX:
      return _T("ADMXRC3_INVALID_INDEX");

    case ADMXRC3_INVALID_REGION:
      return _T("ADMXRC3_INVALID_REGION");

    case ADMXRC3_NULL_POINTER:
      return _T("ADMXRC3_NULL_POINTER");

    case ADMXRC3_BUSY:
      return _T("ADMXRC3_BUSY");

    case ADMXRC3_CANCEL_FAILED:
      return _T("ADMXRC3_CANCEL_FAILED");

    case ADMXRC3_CANCELLED:
      return _T("ADMXRC3_CANCELLED");

    case ADMXRC3_RESOURCE_LIMIT:
      return _T("ADMXRC3_RESOURCE_LIMIT");

    case ADMXRC3_REGION_TOO_LARGE:
      return _T("ADMXRC3_REGION_TOO_LARGE");

    case ADMXRC3_MUST_WAIT:
      return _T("ADMXRC3_MUST_WAIT");

    case ADMXRC3_NOT_OWNER:
      return _T("ADMXRC3_NOT_OWNER");

    case ADMXRC3_PENDING:
      return _T("ADMXRC3_PENDING");

    case ADMXRC3_NONBLOCK_IDLE:
      return _T("ADMXRC3_NONBLOCK_IDLE");

    case ADMXRC3_INVALID_BUFFER_HANDLE:
      return _T("ADMXRC3_INVALID_BUFFER_HANDLE");

    case ADMXRC3_INVALID_LOCAL_REGION:
      return _T("ADMXRC3_INVALID_LOCAL_REGION");

    case ADMXRC3_DATATYPE_MISMATCH:
      return _T("ADMXRC3_DATATYPE_MISMATCH");

    default:
      break;
    }
  } else {
    switch (code) {
    case ADMXRC3_SUCCESS:
      return _T("No error");

    case ADMXRC3_INTERNAL_ERROR:
      return _T("An error in the API logic was detected");

    case ADMXRC3_UNEXPECTED_ERROR:
      return _T("An unexpected error occurred");

    case ADMXRC3_BAD_DRIVER:
      return _T("The driver may be incorrectly installed");

    case ADMXRC3_NO_MEMORY:
      return _T("Could not allocate memory required to complete operation");

    case ADMXRC3_ACCESS_DENIED:
      return _T("Caller does not have permission to open the device");

    case ADMXRC3_DEVICE_NOT_FOUND:
      return _T("Failed to open the device with specified CardID");

    case ADMXRC3_FILE_NOT_FOUND:
      return _T("Failed to open the specified bitstream file");

    case ADMXRC3_HARDWARE_ERROR:
      return _T("A hardware error occurred");

    case ADMXRC3_FPGA_MISMATCH:
      return _T("The FPGA on the card did not match that of the bitstream file");

    case ADMXRC3_INVALID_BITSTREAM:
      return _T("The bitstream file was corrupt");

    case ADMXRC3_INVALID_BUFFER:
      return _T("The supplied buffer was invalid");

    case ADMXRC3_INVALID_FLAG:
      return _T("A flag was invalid or not recognized");

    case ADMXRC3_INVALID_FREQUENCY:
      return _T("The frequency was invalid");

    case ADMXRC3_INVALID_HANDLE:
      return _T("The device handle was invalid");

    case ADMXRC3_INVALID_INDEX:
      return _T("The index parameter was invalid");

    case ADMXRC3_INVALID_REGION:
      return _T("The offset and/or length parameter(s) were invalid");

    case ADMXRC3_NULL_POINTER:
      return _T("A NULL pointer was passed where NULL was illegal");

    case ADMXRC3_BUSY:
      return _T("There is already an operation in progress");

    case ADMXRC3_CANCEL_FAILED:
      return _T("There is nothing to cancel");

    case ADMXRC3_CANCELLED:
      return _T("The operation was cancelled");

    case ADMXRC3_RESOURCE_LIMIT:
      return _T("A resource limit was reached");

    case ADMXRC3_REGION_TOO_LARGE:
      return _T("The region is too large for a single operation");

    case ADMXRC3_MUST_WAIT:
      return _T("The operation could not be started immediately");

    case ADMXRC3_NOT_OWNER:
      return _T("The buffer handle belongs to a different device handle");

    case ADMXRC3_PENDING:
      return _T("The non-blocking operation is now in progress");

    case ADMXRC3_NONBLOCK_IDLE:
      return _T("There is no non-blocking operation to finish");

    case ADMXRC3_INVALID_BUFFER_HANDLE:
      return _T("The buffer handle is invalid");

    case ADMXRC3_INVALID_LOCAL_REGION:
      return _T("The local address region is invalid");

    case ADMXRC3_DATATYPE_MISMATCH:
      return _T("The datatype did not match that of the sensor");

    default:
      break;
    }
  }
  _stprintf_s(buffer, ARRAY_LENGTH(buffer), _T("Unknown status code %lu (0x%lX)"), (unsigned long)code, (unsigned long)code);
  return buffer;
}

ADMXRC3_EXPORT ADMXRC3_STATUS
ADMXRC3_CALLING_CONVENTION
ADMXRC3_GetVersionInfo(
  ADMXRC3_HANDLE hCard,
  ADMXRC3_VERSION_INFO* pVersionInfo) {
  unsigned int index;

  if (!bDeviceValid(hCard, index)) {
    return ADMXRC3_INVALID_HANDLE;
  };

  pVersionInfo->Api.Major = 1;
  pVersionInfo->Api.Minor = 2;
  pVersionInfo->Api.Bugfix = 3;
  pVersionInfo->Driver.Major = 4;
  pVersionInfo->Driver.Minor = 5;
  pVersionInfo->Driver.Bugfix = 6;

  return ADMXRC3_SUCCESS;
}

ADMXRC3_EXPORT ADMXRC3_STATUS
ADMXRC3_CALLING_CONVENTION
ADMXRC3_GetWindowInfo(
  ADMXRC3_HANDLE hCard,
  unsigned int windowIndex,
  ADMXRC3_WINDOW_INFO* pWindowInfo) {
  unsigned int index;

  if (!bDeviceValid(hCard, index)) {
    return ADMXRC3_INVALID_HANDLE;
  };

  switch ((index + windowIndex) % 4) {
  case 0:
    pWindowInfo->BusBase = 0x10000000U;
    pWindowInfo->BusSize = 0x800000U;
    pWindowInfo->LocalBase = 0x0;
    pWindowInfo->LocalSize = 0x800000U;
    pWindowInfo->VirtualSize = 0x800000U;
    return ADMXRC3_SUCCESS;

  case 1:
    pWindowInfo->BusBase = 0x20000000U;
    pWindowInfo->BusSize = 0x100000U;
    pWindowInfo->LocalBase = 0x4000000U;
    pWindowInfo->LocalSize = 0x100000U;
    pWindowInfo->VirtualSize = 0x100000U;
    return ADMXRC3_SUCCESS;

  case 2:
    pWindowInfo->BusBase = 0x30000000U;
    pWindowInfo->BusSize = 0x1000U;
    pWindowInfo->LocalBase = 0;
    pWindowInfo->LocalSize = 0;
    pWindowInfo->VirtualSize = 0x1000U;
    return ADMXRC3_SUCCESS;

  case 3:
    return ADMXRC3_UNEXPECTED_ERROR;
  }

  return ADMXRC3_INVALID_INDEX;
}

ADMXRC3_EXPORT ADMXRC3_STATUS
ADMXRC3_CALLING_CONVENTION
ADMXRC3_Open(
  unsigned int index,
  ADMXRC3_HANDLE* phCard) {
  if (index < g_numDevice) {
    if (index == g_numDevice - 1) {
      return ADMXRC3_UNEXPECTED_ERROR;
    } else {
      *phCard = (ADMXRC3_HANDLE)(index + 1);
      return ADMXRC3_SUCCESS;
    }
  } else {
    return ADMXRC3_DEVICE_NOT_FOUND;
  }
}

ADMXRC3_EXPORT ADMXRC3_STATUS
ADMXRC3_CALLING_CONVENTION
ADMXRC3_OpenEx(
  unsigned int index,
  _ADMXRC3_BOOL /* bReadOnly */,
  _ADMXRC3_UINT32 /* cooperativeLevel */,
  ADMXRC3_HANDLE* phCard) {
  return ADMXRC3_Open(index, phCard);
}

ADMXRC3_EXPORT ADMXRC3_STATUS
ADMXRC3_CALLING_CONVENTION
ADMXRC3_GetSensorInfo(
  ADMXRC3_HANDLE hCard,
  unsigned int sensorIndex,
  ADMXRC3_SENSOR_INFO* pSensorInfo) {
  unsigned int index;

  if (!bDeviceValid(hCard, index)) {
    return ADMXRC3_INVALID_HANDLE;
  };

  memset(pSensorInfo, 0, sizeof(*pSensorInfo));
  switch (index) {
  case 0:
    return ADMXRC3_INVALID_INDEX;
    break;

  case 2:
    return ADMXRC3_UNEXPECTED_ERROR;

  case 3:
    _stprintf_s(pSensorInfo->Description, ARRAY_LENGTH(pSensorInfo->Description), _T("Sensor %u"), sensorIndex);
    pSensorInfo->Capabilities = 0;
    pSensorInfo->DataType = (ADMXRC3_DATA_TYPE)((sensorIndex + index) % 4);
    if (pSensorInfo->DataType == ADMXRC3_DATA_BOOL) {
      pSensorInfo->Unit = ADMXRC3_UNIT_NONE;
    } else {
      pSensorInfo->Unit = ADMXRC3_UNIT_V;
    }
    pSensorInfo->Error = (double)(rand() % 1000) / 10000.0;
    pSensorInfo->Exponent = ((sensorIndex + index) % 6) - 6;
    pSensorInfo->Capabilities = (uint32_t)((rand() & 0x3310) >> 4);
    break;

  default:
    if (index == 3 && sensorIndex == 1) {
      return ADMXRC3_UNEXPECTED_ERROR;
    }
    _stprintf_s(pSensorInfo->Description, ARRAY_LENGTH(pSensorInfo->Description), _T("Sensor %u"), sensorIndex);
    pSensorInfo->Capabilities = 0;
    pSensorInfo->DataType = (ADMXRC3_DATA_TYPE)((sensorIndex + index) % 10);
    if (pSensorInfo->DataType == ADMXRC3_DATA_BOOL) {
      pSensorInfo->Unit = ADMXRC3_UNIT_NONE;
    } else {
      pSensorInfo->Unit = (ADMXRC3_UNIT_TYPE)((7 - sensorIndex + index) % 7);
    }
    pSensorInfo->Error = (double)(rand() % 1000) / 10000.0;
    pSensorInfo->Exponent = ((sensorIndex + index) % 6) - 6;
    pSensorInfo->Capabilities = (uint32_t)((rand() & 0x3310) >> 4);
    break;
  }

  return ADMXRC3_SUCCESS;

}

ADMXRC3_EXPORT ADMXRC3_STATUS
ADMXRC3_CALLING_CONVENTION
ADMXRC3_ReadSensor(
  ADMXRC3_HANDLE hCard,
  unsigned int sensorIndex,
  ADMXRC3_SENSOR_VALUE* pValue) {
  unsigned int index;
  ADMXRC3_STATUS status;
  ADMXRC3_SENSOR_INFO sensorInfo;

  if (!bDeviceValid(hCard, index)) {
    return ADMXRC3_INVALID_HANDLE;
  };

  status = ADMXRC3_GetSensorInfo(hCard, sensorIndex, &sensorInfo);
  if (ADMXRC3_SUCCESS != status) {
    return status;
  }

  memset(pValue, 0, sizeof(*pValue));

  switch (sensorInfo.DataType) {
  case ADMXRC3_DATA_BOOL:
    pValue->Bool = (rand() & 0x100) ? TRUE : FALSE;
    break;

  case ADMXRC3_DATA_DOUBLE:
    pValue->Double = (double)(rand() % 10000) * 0.001;
    break;

  case ADMXRC3_DATA_INT32:
    pValue->Int32 = (rand() % 200) - 89;
    break;

  case ADMXRC3_DATA_UINT32:
    pValue->UInt32 = (unsigned int)(rand() % 100) + 27U;
    break;

  default:
    return ADMXRC3_UNEXPECTED_ERROR;
  }

  return ADMXRC3_SUCCESS;
}
