--==============================================================================
--
--     File Name: blk_chipscope.vhd
--       Version: 0.2
--          Date: 09 Feb 2011
--         Model: synth
--
--  Dependencies:
--
--       Company: Alpha Data
--        Design: chipscope instantiation option
--
--   Description: Adds chipscope connections to OCP channels
--
--   Limitations:
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2010-2011 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 12 Mar 2010: Initial design
--                0.2, TW, 09 Feb 2011: Added "instantiate" generic, so that
--                  chipscope core instantiation can easily be disabled.
--
--==============================================================================

-- Intstantiation:

--  ----------------------------------------
--  -- Chipscope OCP interface connection --
--  ----------------------------------------
--
--  blk_chipscope_i : blk_chipscope
--  generic map(
--    instantiate => CHIPSCOPE_ON)  -- boolean := true
--  port map(
--    -- OCP channel 0 interface
--    ocp_ch0_clk   => ,          -- in    std_logic
--    ocp_ch0_m2s   => ,          -- in    adb3_ocp_m2sT
--    ocp_ch0_s2m   => ,          -- in    adb3_ocp_s2mT
--    ocp_ch0_trig  => ,          -- in    std_logic
--    -- OCP channel 1 interface
--    ocp_ch1_clk   => ,          -- in    std_logic
--    ocp_ch1_m2s   => ,          -- in    adb3_ocp_m2sT
--    ocp_ch1_s2m   => ,          -- in    adb3_ocp_s2mT
--    ocp_ch1_trig  => ,          -- in    std_logic
--    -- OCP channel 2 interface
--    ocp_ch2_clk   => ,          -- in    std_logic
--    ocp_ch2_m2s   => ,          -- in    adb3_ocp_m2sT
--    ocp_ch2_s2m   => ,          -- in    adb3_ocp_s2mT
--    ocp_ch2_trig  => );         -- in    std_logic)


library ieee;
use ieee.std_logic_1164.all;

library work;
use work.adb3_ocp.all;

entity blk_chipscope is
generic(
  instantiate : boolean := true);
port(
  -- OCP channel 0 interface
  ocp_ch0_clk  : in    std_logic;
  ocp_ch0_m2s  : in    adb3_ocp_m2sT;
  ocp_ch0_s2m  : in    adb3_ocp_s2mT;
  ocp_ch0_trig : in    std_logic_vector(7 downto 0);
  -- OCP channel 1 interface
  ocp_ch1_clk  : in    std_logic;
  ocp_ch1_m2s  : in    adb3_ocp_m2sT;
  ocp_ch1_s2m  : in    adb3_ocp_s2mT;
  ocp_ch1_trig : in    std_logic_vector(7 downto 0);
  -- OCP channel 2 interface
  ocp_ch2_clk  : in    std_logic;
  ocp_ch2_m2s  : in    adb3_ocp_m2sT;
  ocp_ch2_s2m  : in    adb3_ocp_s2mT;
  ocp_ch2_trig : in    std_logic_vector(7 downto 0));
end;


architecture synth of blk_chipscope is

  signal ch0_control  : std_logic_vector(35 downto 0);
  signal ch1_control  : std_logic_vector(35 downto 0);
  signal ch2_control  : std_logic_vector(35 downto 0);

  signal ch0_trig0    : std_logic_vector(7 downto 0);
  signal ch0_trig1    : std_logic_vector(7 downto 0);
  signal ch1_trig0    : std_logic_vector(7 downto 0);
  signal ch1_trig1    : std_logic_vector(7 downto 0);
  signal ch2_trig0    : std_logic_vector(7 downto 0);
  signal ch2_trig1    : std_logic_vector(7 downto 0);

  signal ch0_data     : std_logic_vector(383 downto 0);
  signal ch1_data     : std_logic_vector(383 downto 0);
  signal ch2_data     : std_logic_vector(383 downto 0);

--  signal ch0_trig_out : std_logic;
--  signal ch1_trig_out : std_logic;
--  signal ch2_trig_out : std_logic;

  component chipscope_icon
  port (
    CONTROL0 : inout std_logic_vector(35 downto 0);
    CONTROL1 : inout std_logic_vector(35 downto 0);
    CONTROL2 : inout std_logic_vector(35 downto 0));
  end component;

  component chipscope_ila
  PORT (
    CLK      : in    std_logic;
    CONTROL  : inout std_logic_vector(35 downto 0);
    DATA     : in    std_logic_vector(383 downto 0);
    TRIG0    : in    std_logic_vector(7 downto 0);
    TRIG1    : in    std_logic_vector(7 downto 0);
    TRIG_OUT : out   std_logic);
  end component;

begin

  instance : if instantiate generate
    -- Control
    chipscope_icon_i : chipscope_icon
    port map (
      CONTROL0 => ch0_control,
      CONTROL1 => ch1_control,
      CONTROL2 => ch2_control);

    -- Chipscope channel 0: OCP channel 0
    ch0_trig0(1 downto 0) <= ocp_ch0_m2s.Cmd(1 downto 0);
    ch0_trig0(2)          <= ocp_ch0_m2s.DataValid;
    ch0_trig0(3)          <= ocp_ch0_m2s.RespAccept;
    ch0_trig0(4)          <= ocp_ch0_s2m.CmdAccept;
    ch0_trig0(5)          <= ocp_ch0_s2m.DataAccept;
    ch0_trig0(7 downto 6) <= ocp_ch0_s2m.Resp;

    ch0_trig1 <= ocp_ch0_trig;

    ch0_data(127 downto 0)   <= ocp_ch0_m2s.Data;
    ch0_data(255 downto 128) <= ocp_ch0_s2m.Data;
    ch0_data(275 downto 256) <= ocp_ch0_m2s.Addr(19 downto 0);
    ch0_data(287 downto 276) <= ocp_ch0_m2s.BurstLength;
    ch0_data(303 downto 288) <= ocp_ch0_m2s.DataByteEn;
    ch0_data(311 downto 304) <= ocp_ch0_m2s.Tag;
    ch0_data(319 downto 312) <= ocp_ch0_s2m.Tag;
    ch0_data(327 downto 320) <= ch0_trig0;
    ch0_data(335 downto 328) <= ch0_trig1;
    ch0_data(355 downto 336) <= ocp_ch0_m2s.Addr(39 downto 20);
    ch0_data(383 downto 356) <= (others => '0');

    chipscope_ila_ch0_i : chipscope_ila
    port map (
      CONTROL  => ch0_control,
      CLK      => ocp_ch0_clk,
      DATA     => ch0_data,
      TRIG0    => ch0_trig0,
      TRIG1    => ch0_trig1,
      TRIG_OUT => open);

    -- Chipscope channel 1: OCP channel 1
    ch1_trig0(1 downto 0) <= ocp_ch1_m2s.Cmd(1 downto 0);
    ch1_trig0(2)          <= ocp_ch1_m2s.DataValid;
    ch1_trig0(3)          <= ocp_ch1_m2s.RespAccept;
    ch1_trig0(4)          <= ocp_ch1_s2m.CmdAccept;
    ch1_trig0(5)          <= ocp_ch1_s2m.DataAccept;
    ch1_trig0(7 downto 6) <= ocp_ch1_s2m.Resp;

    ch1_trig1 <= ocp_ch1_trig;

    ch1_data(127 downto 0)   <= ocp_ch1_m2s.Data;
    ch1_data(255 downto 128) <= ocp_ch1_s2m.Data;
    ch1_data(275 downto 256) <= ocp_ch1_m2s.Addr(19 downto 0);
    ch1_data(287 downto 276) <= ocp_ch1_m2s.BurstLength;
    ch1_data(303 downto 288) <= ocp_ch1_m2s.DataByteEn;
    ch1_data(311 downto 304) <= ocp_ch1_m2s.Tag;
    ch1_data(319 downto 312) <= ocp_ch1_s2m.Tag;
    ch1_data(327 downto 320) <= ch1_trig0;
    ch1_data(335 downto 328) <= ch1_trig1;
    ch1_data(355 downto 336) <= ocp_ch1_m2s.Addr(39 downto 20);
    ch1_data(383 downto 356) <= (others => '0');

    chipscope_ila_ch1_i : chipscope_ila
    port map (
      CONTROL  => ch1_control,
      CLK      => ocp_ch1_clk,
      DATA     => ch1_data,
      TRIG0    => ch1_trig0,
      TRIG1    => ch1_trig1,
      TRIG_OUT => open);

    -- Chipscope channel 2: OCP channel 2
    ch2_trig0(1 downto 0) <= ocp_ch2_m2s.Cmd(1 downto 0);
    ch2_trig0(2)          <= ocp_ch2_m2s.DataValid;
    ch2_trig0(3)          <= ocp_ch2_m2s.RespAccept;
    ch2_trig0(4)          <= ocp_ch2_s2m.CmdAccept;
    ch2_trig0(5)          <= ocp_ch2_s2m.DataAccept;
    ch2_trig0(7 downto 6) <= ocp_ch2_s2m.Resp;

    ch2_trig1 <= ocp_ch2_trig;

    ch2_data(127 downto 0)   <= ocp_ch2_m2s.Data;
    ch2_data(255 downto 128) <= ocp_ch2_s2m.Data;
    ch2_data(275 downto 256) <= ocp_ch2_m2s.Addr(19 downto 0);
    ch2_data(287 downto 276) <= ocp_ch2_m2s.BurstLength;
    ch2_data(303 downto 288) <= ocp_ch2_m2s.DataByteEn;
    ch2_data(311 downto 304) <= ocp_ch2_m2s.Tag;
    ch2_data(319 downto 312) <= ocp_ch2_s2m.Tag;
    ch2_data(327 downto 320) <= ch2_trig0;
    ch2_data(335 downto 328) <= ch2_trig1;
    ch2_data(355 downto 336) <= ocp_ch2_m2s.Addr(39 downto 20);
    ch2_data(383 downto 356) <= (others => '0');

    chipscope_ila_ch2_i : chipscope_ila
    port map (
      CONTROL  => ch2_control,
      CLK      => ocp_ch2_clk,
      DATA     => ch2_data,
      TRIG0    => ch2_trig0,
      TRIG1    => ch2_trig1,
      TRIG_OUT => open);
  end generate;

end;