--==============================================================================
--
--     File Name: mptl128_interface_bridge_6tl_sim.vhd
--       Version: 0.1
--          Date: 11 May 2010
--         Model: sim (Not Synthesisable)
--
--  Dependencies:
--
--       Company: Alpha Data
--        Design: bridge mptl interface wrapper
--
--   Description: Wrapper for full bridge mptl interface simualtion netlist
--
--   Limitations:
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2010 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 11 May 2010: Initial design
--
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;

library work;
use work.adb3_ocp.all;
use work.adb3_target_inc_pkg.all;

entity mptl64par_interface_bridge_6tl is
port(
  -- OCP Interface
  ocp_clk           : in    std_logic;
  direct_slave_m2s  : in    adb3_ocp_m2sT;
  direct_slave_s2m  : out   adb3_ocp_s2mT;
  dma_channels_m2s  : in    adb3_ocp_m2sT_array(0 to 1);
  dma_channels_s2m  : out   adb3_ocp_s2mT_array(0 to 1);
  direct_master_m2s : out   adb3_ocp_m2sT;
  direct_master_s2m : in    adb3_ocp_s2mT;
  -- MPTL Interface
  mptl_tx           : out   mptl_pins_t;
  mptl_rx           : in    mptl_pins_t;
  mptl_clk          : in    std_logic_dbl_t;
  -- Bridge to Target Sideband Signals
  target_configured : in    std_logic;
  target_gtp_online : in    std_logic;
  bridge_gtp_online : out   std_logic;
  -- MPTL64 Sideband Interface
  gpio_b2t          : in    std_logic_vector(15 downto 0);
  gpio_t2b          : out   std_logic_vector(15 downto 0);
  mptl_status       : out   std_logic_vector(63 downto 0);
  debug             : out   std_logic_vector(255 downto 0));
end;

architecture rtl of mptl64par_interface_bridge_6tl is

  signal direct_slave_m2s_i  : std_logic_vector(adb3_ocp_m2sT_slv_length-1 downto 0);
  signal direct_slave_s2m_i  : std_logic_vector(adb3_ocp_s2mT_slv_length-1 downto 0);
  signal dma_channels_m2s_i  : std_logic_vector(2*adb3_ocp_m2sT_slv_length-1 downto 0);
  signal dma_channels_s2m_i  : std_logic_vector(2*adb3_ocp_s2mT_slv_length-1 downto 0);
  signal direct_master_m2s_i : std_logic_vector(adb3_ocp_m2sT_slv_length-1 downto 0);
  signal direct_master_s2m_i : std_logic_vector(adb3_ocp_s2mT_slv_length-1 downto 0);

begin

  direct_slave_m2s_i <= to_std_logic_vector(direct_slave_m2s);
  direct_slave_s2m <= to_adb3_ocp_s2mT(direct_slave_s2m_i);

  dma_channels_m2s_i <= to_std_logic_vector(dma_channels_m2s);
  dma_channels_s2m <= to_adb3_ocp_s2mT_array(dma_channels_s2m_i);

  direct_master_m2s <= to_adb3_ocp_m2sT(direct_master_m2s_i);
  direct_master_s2m_i <= to_std_logic_vector(direct_master_s2m);

  mptl64par_interface_bridge_6tl_1: entity work.mptl64par_interface_bridge_6tl_slv
    port map (
      ocp_clk            => ocp_clk,
      direct_slave_m2s   => direct_slave_m2s_i,
      direct_slave_s2m   => direct_slave_s2m_i,
      dma_channels_m2s   => dma_channels_m2s_i,
      dma_channels_s2m   => dma_channels_s2m_i,
      direct_master_m2s  => direct_master_m2s_i,
      direct_master_s2m  => direct_master_s2m_i,
      mptl_tx_d          => mptl_tx.d(31 downto 0),
      mptl_tx_v          => mptl_tx.v,
      mptl_tx_ck         => mptl_tx.ck,
      mptl_rx_d          => mptl_rx.d(31 downto 0),
      mptl_rx_v          => mptl_rx.v,
      mptl_rx_ck         => mptl_rx.ck,
      mptl_clk_p         => mptl_clk.p,
      mptl_clk_n         => mptl_clk.n,
      target_configured  => target_configured,
      target_gtp_online  => target_gtp_online,
      bridge_gtp_online  => bridge_gtp_online,
      gpio_b2t           => gpio_b2t,
      gpio_t2b           => gpio_t2b,
      mptl_status        => mptl_status,
      debug              => debug);

end rtl;