--==============================================================================
--
--     File Name: mptl64par_interface_bridge_6tl_top.vhd
--       Version: 0.1
--          Date: 11 May 2010
--         Model: sim (Not Synthesisable)
--
--  Dependencies:
--
--       Company: Alpha Data
--        Design: target mptl interface wrapper
--
--   Description: Selects between the synthesis target mptl interfaces
--
--   Limitations:
--
--         Notes:
--
--    Disclaimer: THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY
--                WHATSOEVER AND ALPHA DATA SPECIFICALLY DISCLAIMS ANY
--                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
--                Copyright (C) 2010 Alpha Data, All rights reserved
--
--     Revisions: 0.1, KDC, 11 May 2010: Initial design
--
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;

library work;
use work.adb3_ocp.all;
use work.adb3_target_types_pkg.all;
use work.adb3_target_inc_pkg.all;
use work.adb3_target_pkg.all;
use work.adb3_target_tb_pkg.all;

entity mptl64par_interface_bridge_6tl_top is
port(
  -- OCP Interface
  ocp_clk_in         : in    std_logic;
  direct_slave_m2s   : in    adb3_ocp_m2sT;
  direct_slave_s2m   : out   adb3_ocp_s2mT;
  dma_channels_m2s   : in    adb3_ocp_m2sT_array(0 to DMA_CHANNELS-1);
  dma_channels_s2m   : out   adb3_ocp_s2mT_array(0 to DMA_CHANNELS-1);
  direct_masters_m2s : out   adb3_ocp_m2sT_array(0 to DM_CHANNELS-1);
  direct_masters_s2m : in    adb3_ocp_s2mT_array(0 to DM_CHANNELS-1);
  dma_abort          : out   std_logic_vector(DMA_CHANNELS-1 downto 0);
  ocp_clk_out        : out   std_logic;
  -- MPTL Interface
  mptl_t2b           : in    mptl_pins_t;
  mptl_b2t           : out   mptl_pins_t;
  mptl_clk           : in    std_logic_dbl_t;
  -- MPTL Initialisation
  mptl_sb_t2b        : in    mptl_sb_t2b_t;
  mptl_sb_b2t        : out   mptl_sb_b2t_t;
  -- MPTL Sideband
  gpio_b2t           : in    std_logic_vector(15 downto 0);
  gpio_t2b           : out   std_logic_vector(15 downto 0);
  mptl_status        : out   std_logic_vector(63 downto 0);
  mptl_config        : in    std_logic_vector(31 downto 0));
end;


architecture synth of mptl64par_interface_bridge_6tl_top is

  signal dm_int_m2s : adb3_ocp_m2sT_array(0 to 0);
  signal dm_int_s2m : adb3_ocp_s2mT_array(0 to 0);

  signal target_configured : std_logic;
  signal target_gtp_online : std_logic;
  signal bridge_gtp_online : std_logic;

begin

  sim_mptl_g : if (TARGET_USE = SIM_MPTL) generate
    -- Define data during MPTL simulation
    dm_g : for n in 0 to DM_CHANNELS-1 generate
      direct_masters_m2s(n) <= dm_int_m2s(n);
      dm_int_s2m(n)         <= make_defined_s2m(direct_masters_s2m(n));
    end generate;
  end generate;


  -- Connect MPTL sideband inputs
  target_configured <= not(mptl_sb_t2b.mptl_target_configured_l);
  target_gtp_online <= not(mptl_sb_t2b.mptl_target_gtp_online_l);

  -- dma_abort obtained directly from pins on 6TL
  dma_abort(1 downto 0) <= not(mptl_sb_t2b.mptl_dma_abort_l);

  -- full OCP-MPTL-OCP interface
  mptl64par_interface_bridge_6tl_i : mptl64par_interface_bridge_6tl
  port map(
    -- OCP Interface
    ocp_clk            => ocp_clk_in,               -- in    std_logic
    direct_slave_m2s   => direct_slave_m2s,         -- in    adb3_ocp_m2sT
    direct_slave_s2m   => direct_slave_s2m,         -- out   adb3_ocp_s2mT
    dma_channels_m2s   => dma_channels_m2s(0 to 1), -- in    adb3_ocp_m2sT_array(0 to 1)
    dma_channels_s2m   => dma_channels_s2m(0 to 1), -- out   adb3_ocp_s2mT_array(0 to 1)
    direct_master_m2s  => dm_int_m2s(0),            -- out   adb3_ocp_m2sT
    direct_master_s2m  => dm_int_s2m(0),            -- in    adb3_ocp_s2mT
    -- MPTL Interface
    mptl_tx            => mptl_b2t,                 -- out   mptl_pins_t;
    mptl_rx            => mptl_t2b,                 -- in    mptl_pins_t;
    mptl_clk           => mptl_clk,                 -- in    std_logic_dbl_t;
    -- MPTL Sideband Signals
    target_configured  => target_configured,        -- in    std_logic
    target_gtp_online  => target_gtp_online,        -- in    std_logic
    bridge_gtp_online  => bridge_gtp_online,        -- out   std_logic
    -- MPTL Sideband Interface
    gpio_b2t           => gpio_b2t,                 -- in    std_logic_vector(15 downto 0)
    gpio_t2b           => gpio_t2b,                 -- out   std_logic_vector(15 downto 0)
    mptl_status        => mptl_status,              -- out   std_logic_vector(63 downto 0)
    debug              => open);                    -- out   std_logic_vector(255 downto 0))

  -- Connect MPTL sideband ouputs
  mptl_sb_b2t.mptl_bridge_gtp_online_l <= not(bridge_gtp_online);

  -- Bridge ocp clock can be different from target ocp clock
  ocp_clk_out <= ocp_clk_in;

end;